<?php
    include 'config.php';
    
    error_reporting(E_ALL);
    date_default_timezone_set("UTC");
    
    if ( ! isset($_POST["SAMLResponse"])) {
        header('HTTP/1.1 400 Bad Request', true, 400);
        exit;
    }
    
    function GetRequestBody() {
        $body = '';
        $fh = @fopen('php://input', 'r');
        if ($fh) {
            while ( ! feof($fh)) {
                $s = fread($fh, 1024);
                if (is_string($s)) {
                    $body .= $s;
                }
            }
        }
        fclose($fh);
        Return $body;
    }
    
    session_start();
    
    if (!extension_loaded("sbb") && !function_exists('SBUtils\SetLicenseKey')) {
        print "SecureBlackbox extension NOT available";
        die;
    }
        
    try {
        if (!SBUtils\GetLicense()) {
          #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it in "sbb.license_key" php.ini directive or here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
          SBUtils\SetLicenseKey("<enter license key here>");
        }
    } catch (SBException $e) {
        print "<p style='color:red'>Setting SBB License Key failed with message: <br/>" . $e->getErrorMessage() . "</p>";
        die;
    }
    
    $binding = new TElSAMLPOSTBinding();
    $binding->set_Mode(TSBSAMLPOSTBindingMode::spmServer);
    
    $body = GetRequestBody();
        
    $saml = $binding->LoadResponse($body);
    $state = $binding->get_RelayState();
    
    $ms = new TMemoryStream();
    $ms->Write($saml, strlen($saml));
    $ms->set_Position(0);
    
    $doc = new TElXMLDOMDocument();
    $doc->LoadFromStream($ms);
    $root = $doc->get_DocumentElement();
    
    if (strcmp($root->get_LocalName(), 'Response') !== 0) {
        header('HTTP/1.1 400 Bad Request', true, 400);
        exit;
    }
    
    $idpCert = new TElX509Certificate(NULL);
    $idpCert->LoadFromFileAuto(IDP_SIGN_CERT, "");
    
    $sigHandler = new TElSAMLSignatureHandler();
    $sigHandler->set_Certificate($idpCert);
    $sigHandler->set_ValidateReferences(true);
    
    $samlResp = new TElSAMLResponseElement();
    $samlResp->set_Handler($sigHandler);
    $samlResp->LoadFromXML($root);
    
    // check response
    if (strcmp($samlResp->get_Version(), SBXMLSAMLCore\SAMLVersion) !== 0) {
        header('HTTP/1.1 400 Bad Request', true, 400);
        exit;
    }
    
    $diff = SBUtils\DateTimeToUnixTime($samlResp->get_IssueInstant()) - SBUtils\DateTimeToUnixTime(SBUtils\UTCNow());
    if ($diff > MAX_ISSUE_INSTANT_TIME_DIFF) {
        header('HTTP/1.1 400 Bad Request', true, 400);
        exit;
    }
    
    $reqID2 = $samlResp->get_InResponseTo();
    if (isset($_SESSION['samlReqID']))
        $reqID1 = $_SESSION['samlReqID'];
    else
        $reqID1 = '';
    
    if (strlen($reqID1) > 0 && strlen($reqID2) > 0 and strcmp($reqID2, $reqID1) !== 0) {
        header('HTTP/1.1 400 Bad Request', true, 400);
        exit;
    }
    
    $statusCode = $samlResp->get_Status()->get_StatusCode()->get_Value();
    if (strcmp($statusCode, SBXMLSAMLProtocol\xmlSAMLStatusSuccess) !== 0) {
        header('HTTP/1.1 400 Bad Request', true, 400);
        exit;
    }
    
    for ($i = 0; $i < $samlResp->get_AssertionCount(); $i++) {
        $at = $samlResp->get_Assertions($i);
        
        if ($at->ClassName_Inst() == 'TElSAMLEncryptedAssertionElement') {
            // encrypted assertion
            $ea = $at->CastTo();
            
            $encCert = new TElX509Certificate();
            $encCert->LoadFromFileAuto(SP_ENCRYPTION_CERT, "");
            
            $encHandler = new TElSAMLEncryptionHandler();
            $encHandler->set_Certificate($encCert);
            
            $ea->set_Handler($encHandler);
            $ea->Decrypt();
            
            if ($ea->get_Decrypted()) {
                $assertion = $ea->CastTo();
            } else {
                header('HTTP/1.1 500 Internal Error', true, 500);
                exit;
            }              
        } else {
            $assertion = $at->CastTo();
        }
        
        for ($j = 0; $j < $assertion->get_StatementCount(); $j++) {
            if ($assertion->get_Statements($j)->ClassName_Inst() == 'TElSAMLAuthnStatementElement') {
                $statement = $assertion->get_Statements($j);
                // check statement here
            }
        }
    }
    
    $_SESSION['auth'] = true;
    header('HTTP/1.1 302 Moved Temporarily', true, 302);
    header('Location: ' . $state);
?>
